use nom::branch::alt;
use nom::bytes::complete::tag;
use nom::bytes::complete::take_until;
use nom::combinator::map;
use nom::combinator::value;
use nom::sequence::delimited;
use nom::IResult;

#[derive(Clone, Debug)]
enum DustTag<'a> {
    DTSpecial(Special),
    DTComment(Comment<'a>),
}

#[derive(Clone, Debug)]
enum Special {
    Space,
    NewLine,
    CarriageReturn,
    LeftCurlyBrace,
    RightCurlyBrace,
}

#[derive(Clone, Debug)]
struct Comment<'a> {
    value: &'a str,
}

fn dust_tag(i: &str) -> IResult<&str, DustTag> {
    alt((
        map(special, DustTag::DTSpecial),
        map(comment, DustTag::DTComment),
    ))(i)
}

fn special(i: &str) -> IResult<&str, Special> {
    delimited(
        tag("{~"),
        alt((
            value(Special::Space, tag("s")),
            value(Special::NewLine, tag("s")),
            value(Special::CarriageReturn, tag("s")),
            value(Special::LeftCurlyBrace, tag("s")),
            value(Special::RightCurlyBrace, tag("s")),
        )),
        tag("}"),
    )(i)
}

fn comment(i: &str) -> IResult<&str, Comment> {
    map(delimited(tag("{!"), take_until("!}"), tag("!}")), |body| {
        Comment { value: body }
    })(i)
}
